@@ -23,7 +23,11 @@ def Cd_cylinder(
     csuph = 9.9999999999999787
     csupscl = -0.4570690347113859
 
-    x = np.log10(Re_D)
+    x = np.where(
+        Re_D == 0,
+        0,
+        np.log10(np.abs(Re_D))
+    )
 
     if subcritical_only:
         Cd = 10 ** (csub0 * x + csub1) + csub2 + csub3 * x
@@ -79,6 +83,10 @@ def Cf_flat_plate(
             * "hybrid-sharpe-nonconvex": A hybrid model that blends the Blasius and Cengel models. Nonconvex in
             log-log-space; however, it may capture some truly nonconvex behavior near transitional Reynolds numbers.
 
+    Returns:
+
+        C_f: The skin friction coefficient, normalized to the length of the plate.
+
     You can view all of these functions graphically using
     `aerosandbox.library.aerodynamics.test_aerodynamics.test_Cf_flat_plate.py`
 
