@@ -1,3 +1,7 @@
+"""
+A set of tools used for making prettier Matplotlib plots.
+"""
+
 import matplotlib.pyplot as plt
 import seaborn as sns
 import matplotlib as mpl
@@ -28,7 +32,7 @@ sns.set_theme(
 
 mpl.rcParams["figure.dpi"] = 200
 mpl.rcParams["axes.formatter.useoffset"] = False
-
+mpl.rcParams["contour.negative_linestyle"] = 'solid'
 
 def set_ticks(
         x_major: Union[float, int] = None,
@@ -46,6 +50,8 @@ def set_ticks(
     if y_minor is not None:
         ax.yaxis.set_minor_locator(ticker.MultipleLocator(base=y_minor))
 
+def equal():
+    plt.gca().set_aspect("equal", adjustable='box')
 
 def adjust_lightness(color, amount=1.0):
     """
@@ -143,7 +149,8 @@ def contour(
         linewidths=0.5,
         extendrect=True,
         linelabels_format: Union[str, Callable[[float], str]] = eng_string,
-        linelabels_fontsize=10,
+        linelabels_fontsize=8,
+        colorbar_label=None,
         contour_kwargs: Dict = None,
         contourf_kwargs: Dict = None,
         colorbar_kwargs: Dict = None,
@@ -207,6 +214,9 @@ def contour(
     shared_kwargs["alpha"] = alpha
     shared_kwargs["extend"] = extend
 
+    if colorbar_label is not None:
+        colorbar_kwargs["label"] = colorbar_label
+
     contour_kwargs = {
         "colors"    : linecolor,
         "linewidths": linewidths,
