@@ -1,29 +1,45 @@
 import math
-import aerosandbox.numpy as np
 import hashlib
 
 
-def eng_string(x: float, format='%.3g', si=True) -> str:
+def eng_string(
+        x: float,
+        unit: str = None,
+        format='%.3g',
+        si=True
+) -> str:
     '''
     Taken from: https://stackoverflow.com/questions/17973278/python-decimal-engineering-notation-for-mili-10e-3-and-micro-10e-6/40691220
 
     Returns float/int value <x> formatted in a simplified engineering format -
     using an exponent that is a multiple of 3.
 
-    format: printf-style string used to format the value before the exponent.
+    Args:
 
-    si: if true, use SI suffix for exponent, e.g. k instead of e3, n instead of
-    e-9 etc.
+        x: The value to be formatted. Float or int.
 
-    E.g. with format='%.2f':
+        unit: A unit of the quantity to be expressed, given as a string. Example: Newtons -> "N"
+
+        format: A printf-style string used to format the value before the exponent.
+
+        si: if true, use SI suffix for exponent. (k instead of e3, n instead of
+            e-9, etc.)
+
+    Examples:
+
+    With format='%.2f':
         1.23e-08 -> 12.30e-9
              123 -> 123.00
           1230.0 -> 1.23e3
       -1230000.0 -> -1.23e6
 
-    and with si=True:
+    With si=True:
           1230.0 -> "1.23k"
       -1230000.0 -> "-1.23M"
+
+    With unit="N" and si=True:
+          1230.0 -> "1.23 kN"
+      -1230000.0 -> "-1.23 MN"
     '''
     sign = ''
     if x < 0:
@@ -36,16 +52,25 @@ def eng_string(x: float, format='%.3g', si=True) -> str:
     x3 = x / (10 ** exp3)
 
     if si and exp3 >= -24 and exp3 <= 24 and exp3 != 0:
-        exp3_text = 'yzafpnum kMGTPEZY'[(exp3 + 24) // 3]
+        exp3_text = 'yzafpnμm kMGTPEZY'[(exp3 + 24) // 3]
     elif exp3 == 0:
         exp3_text = ''
     else:
         exp3_text = 'e%s' % exp3
 
+    if unit is not None:
+        if si:
+            exp3_text = " " + exp3_text + unit
+        else:
+            exp3_text = exp3_text + " " + unit
+
     return ('%s' + format + '%s') % (sign, x3, exp3_text)
 
 
-def latex_sci_notation_string(x: float):
+def latex_sci_notation_string(
+        x: float,
+        format='%.2e',
+):
     """
     Converts a floating-point number to a LaTeX-style formatted string. Does not include the `$$` wrapping to put you in math mode.
 
@@ -53,19 +78,22 @@ def latex_sci_notation_string(x: float):
 
     Examples:
 
-        "
+        latex_sci_notation_string(3000) -> '3 \\times 10^{3}'
     """
-    float_str = "{0:.2g}".format(x)
-    if "e" in float_str:
-        base, exponent = float_str.split("e")
-        return r"{0} \times 10^{{{1}}}".format(base, int(exponent))
+    float_str = format % x
+    base, exponent = float_str.split("e")
+    exponent = int(exponent)
+    if exponent == 0:
+        return base
     else:
-        return float_str
+        return r"{0} \times 10^{{{1}}}".format(base, exponent)
 
 
 def hash_string(string: str) -> int:
     """
     Hashes a string into a quasi-random 32-bit integer! (Based on an MD5 checksum algorithm.)
+
+    Usual warnings apply: it's MD5, don't use this for anything intended to be cryptographically secure.
     """
     md5 = hashlib.md5(string.encode('utf-8'))
     hash_hex = md5.hexdigest()
